.dialog {
	box-shadow:
		0 0px 0px 1px rgba(0,0,0,0.02),
		0 1px 2px rgba(0,0,0,0.05),
		0 2px 4px rgba(0,0,0,0.05),
		0 4px 8px rgba(0,0,0,0.05),
		0 8px 16px rgba(0,0,0,0.05),
		0 16px 32px rgba(0,0,0,0.05),
		0 32px 64px rgba(0,0,0,0.05);
	border-radius: 3px;
	border: 0;
	width: calc(100% - 100px);
	height: calc(100% - 100px);
	padding: 0;
	overflow: hidden;
	position: fixed;
}

.dialog::backdrop {
	background: rgba(0, 0, 0, 0.8);
}

.dialog__close {
	position: absolute;
	right: 5px;
	top: 5px;
	padding: 5px;
	border: 0;
	background: transparent;
	outline: 0;
	cursor: pointer;
	box-shadow: 0px 0px 10px #ffffff;
	background: #fff;
}

.dialog__close svg {
	display: block;
}

.dialog__iframe {
	border: 0;
	position: absolute;
	width: 100%;
	height: 100%;
}
